const fs = require('fs-extra');
const inquirer = require('inquirer');
const graphql = require('graphql');
const path = require('path');
const { RelationalDBSchemaTransformer } = require('graphql-relational-schema-transformer');
const { RelationalDBTemplateGenerator, AuroraServerlessMySQLDatabaseReader } = require('graphql-relational-schema-transformer');
const { mergeTypeDefs } = require('@graphql-tools/merge');
const subcommand = 'add-graphql-datasource';
const categories = 'categories';
const category = 'api';
const providerName = 'awscloudformation';
const rdsRegion = 'rdsRegion';
const rdsIdentifier = 'rdsClusterIdentifier';
const rdsSecretStoreArn = 'rdsSecretStoreArn';
const rdsDatabaseName = 'rdsDatabaseName';
const rdsResourceName = 'rdsResourceName';
const rdsDatasource = 'rdsDatasource';
const rdsInit = 'rdsInit';
module.exports = {
    name: subcommand,
    run: async (context) => {
        const { amplify } = context;
        const servicesMetadata = require('../../provider-utils/supported-datasources').supportedDatasources;
        let resourceName;
        let datasource;
        let databaseName;
        const AWS = await getAwsClient(context, 'list');
        return datasourceSelectionPrompt(context, servicesMetadata)
            .then(result => {
            datasource = result.datasource;
            const providerController = require(`../../provider-utils/${result.providerName}/index`);
            if (!providerController) {
                context.print.error('Provider not configured for this category');
                return;
            }
            return providerController.addDatasource(context, category, result.datasource);
        })
            .then(answers => {
            resourceName = answers.resourceName;
            databaseName = answers.databaseName;
            const currEnv = amplify.getEnvInfo().envName;
            const teamProviderInfoFilePath = amplify.pathManager.getProviderInfoFilePath();
            const teamProviderInfo = context.amplify.readJsonFile(teamProviderInfoFilePath);
            if (!teamProviderInfo[currEnv][categories]) {
                teamProviderInfo[currEnv][categories] = {};
            }
            if (!teamProviderInfo[currEnv][categories][category]) {
                teamProviderInfo[currEnv][categories][category] = {};
            }
            if (!teamProviderInfo[currEnv][categories][category][resourceName]) {
                teamProviderInfo[currEnv][categories][category][resourceName] = {};
            }
            teamProviderInfo[currEnv][categories][category][resourceName][rdsRegion] = answers.region;
            teamProviderInfo[currEnv][categories][category][resourceName][rdsIdentifier] = answers.dbClusterArn;
            teamProviderInfo[currEnv][categories][category][resourceName][rdsSecretStoreArn] = answers.secretStoreArn;
            teamProviderInfo[currEnv][categories][category][resourceName][rdsDatabaseName] = answers.databaseName;
            fs.writeFileSync(teamProviderInfoFilePath, JSON.stringify(teamProviderInfo, null, 4));
            const backendConfigFilePath = amplify.pathManager.getBackendConfigFilePath();
            const backendConfig = context.amplify.readJsonFile(backendConfigFilePath);
            backendConfig[category][resourceName][rdsInit] = true;
            fs.writeFileSync(backendConfigFilePath, JSON.stringify(backendConfig, null, 4));
            const dbReader = new AuroraServerlessMySQLDatabaseReader(answers.region, answers.secretStoreArn, answers.dbClusterArn, answers.databaseName, AWS);
            const relationalSchemaTransformer = new RelationalDBSchemaTransformer(dbReader, answers.databaseName);
            return relationalSchemaTransformer.introspectDatabaseSchema();
        })
            .then(graphqlSchemaContext => {
            const projectBackendDirPath = amplify.pathManager.getBackendDirPath();
            const apiDirPath = `${projectBackendDirPath}/${category}/${resourceName}`;
            fs.ensureDirSync(apiDirPath);
            const graphqlSchemaFilePath = path.join(apiDirPath, 'schema.graphql');
            const schemaFileExists = fs.existsSync(graphqlSchemaFilePath);
            const rdsGraphQLSchemaDoc = graphqlSchemaContext.schemaDoc;
            const schemaDirectoryPath = path.join(apiDirPath, 'schema');
            const schemaDirectoryExists = fs.existsSync(schemaDirectoryPath);
            if (schemaFileExists) {
                const currGraphQLSchemaDoc = readSchema(graphqlSchemaFilePath);
                const concatGraphQLSchemaDoc = mergeTypeDefs([currGraphQLSchemaDoc, rdsGraphQLSchemaDoc], { all: true });
                fs.writeFileSync(graphqlSchemaFilePath, graphql.print(concatGraphQLSchemaDoc), 'utf8');
            }
            else if (schemaDirectoryExists) {
                const rdsSchemaFilePath = path.join(schemaDirectoryPath, 'rds.graphql');
                fs.writeFileSync(rdsSchemaFilePath, graphql.print(rdsGraphQLSchemaDoc), 'utf8');
            }
            else {
                throw new Error(`Could not find a schema in either ${graphqlSchemaFilePath} or schema directory at ${schemaDirectoryPath}`);
            }
            const resolversDir = `${projectBackendDirPath}/${category}/${resourceName}/resolvers`;
            const templateGenerator = new RelationalDBTemplateGenerator(graphqlSchemaContext);
            context[rdsResourceName] = resourceName;
            context[rdsDatasource] = datasource;
            let template = templateGenerator.createTemplate(context);
            template = templateGenerator.addRelationalResolvers(template, resolversDir);
            const cfn = templateGenerator.printCloudformationTemplate(template);
            const stacksDir = `${projectBackendDirPath}/${category}/${resourceName}/stacks`;
            const writeToPath = `${stacksDir}/${resourceName}-${databaseName}-rds.json`;
            fs.writeFileSync(writeToPath, cfn, 'utf8');
            return datasource;
        })
            .then(datasourceName => {
            context.amplify.executeProviderUtils(context, 'awscloudformation', 'compileSchema', { forceCompile: true });
            return datasourceName;
        })
            .then(datasourceName => {
            const { print } = context;
            print.success(`Successfully added the ${datasourceName} datasource locally`);
            print.info('');
            print.success('Some next steps:');
            print.info('"amplify push" will build all your local backend resources and provision it in the cloud');
            print.info('"amplify publish" will build all your local backend and frontend resources (if you have hosting category added) and provision it in the cloud');
            print.info('');
        })
            .catch(err => {
            context.print.info(err.stack);
            context.print.error('There was an error adding the datasource');
            context.usageData.emitError(err);
        });
    },
    readSchema,
};
function datasourceSelectionPrompt(context, supportedDatasources) {
    const options = [];
    Object.keys(supportedDatasources).forEach(datasource => {
        const optionName = supportedDatasources[datasource].alias ||
            `${supportedDatasources[datasource].providerName}:${supportedDatasources[datasource].service}`;
        options.push({
            name: optionName,
            value: {
                provider: supportedDatasources[datasource].provider,
                datasource,
                providerName: supportedDatasources[datasource].provider,
            },
        });
    });
    if (options.length === 0) {
        context.print.error(`No datasources defined by configured providers for category: ${category}`);
        process.exit(1);
    }
    if (options.length === 1) {
        context.print.info(`Using datasource: ${options[0].value.datasource}, provided by: ${options[0].value.providerName}`);
        return new Promise(resolve => {
            resolve(options[0].value);
        });
    }
    const question = [
        {
            name: 'datasource',
            message: 'Please select from one of the below mentioned datasources',
            type: 'list',
            choices: options,
        },
    ];
    return inquirer.prompt(question).then(answer => answer.datasource);
}
async function getAwsClient(context, action) {
    const providerPlugins = context.amplify.getProviderPlugins(context);
    const provider = require(providerPlugins[providerName]);
    return await provider.getConfiguredAWSClient(context, 'aurora-serverless', action);
}
function readSchema(graphqlSchemaFilePath) {
    const graphqlSchemaRaw = fs.readFileSync(graphqlSchemaFilePath).toString();
    let currGraphQLSchemaDoc;
    try {
        currGraphQLSchemaDoc = graphql.parse(graphqlSchemaRaw);
    }
    catch (err) {
        const relativePathToInput = path.relative(process.cwd(), graphqlSchemaRaw);
        throw new Error(`Could not parse graphql schema \n${relativePathToInput}\n${err.message}`);
    }
    return currGraphQLSchemaDoc;
}
//# sourceMappingURL=add-graphql-datasource.js.map